<?php
/**
 * Classe Grisbi
 * 
 * Accs au fichier de comptes au format gsb et traitement
 * @author Nicolas Loeuillet <nicolas.loeuillet@62actu.net>
 * @version 1.0
 * @package core
 * @license http://www.gnu.org/licenses/gpl-3.0.txt GPL version 3.0 license
 */

/**
 * Appel du parseur XML
 */
include('parser.php');

/**
 * 
 * @subpackage classes
 */
class Grisbi
{
  private $xml;
  private $doc;
    
  /**
   * Constructeur qui charge le fichier gsb
   * @param $file Chemin relatif du fichier de comptes
   */
  function __construct($file = '')
  {
    $this->xml = new XMLParser(file_get_contents($file));
    $this->xml->Parse();
    $this->doc = $this->xml->document;
  }
  
  /**
   * Retourne le titre du fichier de comptes
   * @return string le titre du fichier
   */
  public function getTitre()
  {
    return $this->doc->generalites[0]->titre[0]->tagData;
  }
    
  /**
   * Retourne la liste des comptes du fichier
   * @return array tableau avec les comptes
   */
  public function getComptes()
  {
    $comptes = array();
    $compteur = 0;
    foreach($this->doc->comptes[0]->compte as $compte)
    {
      $comptes[$compteur]['id'] = $compte->details[0]->no_de_compte[0]->tagData;
      $comptes[$compteur]['libelle'] = $compte->details[0]->nom[0]->tagData;
      $compteur++;
    }
    return $comptes;
  }

  /**
   * Retourne la liste des comptes sous le solde autoris
   * @return array tableau avec les comptes
   */
  public function getComptesSousSoldeAutorise()
  {
    $comptes = array();
    $compteur = 0;
    foreach($this->doc->comptes[0]->compte as $compte)
    {
    	if ($this->isSousSoldeAutorise($compte->details[0]->no_de_compte[0]->tagData))
    	{
	      $comptes[$compteur]['id'] = $compte->details[0]->no_de_compte[0]->tagData;
	      $comptes[$compteur]['libelle'] = $compte->details[0]->nom[0]->tagData;
    	}
      $compteur++;
    }
    return $comptes;
  }
  
  /**
   * Retourne le solde courant d'un compte donn
   * @param $idCompte identifiant d'un compte, correspond  No_de_compte dans le fichier .gsb
   * @return float le solde courant du compte
   */
  public function getSoldeCourant($idCompte)
  {
    $solde = '';
    foreach($this->doc->comptes[0]->compte as $compte)
    {
      if ($compte->details[0]->no_de_compte[0]->tagData == $idCompte) {
        $solde = $compte->details[0]->solde_courant[0]->tagData;
      }
    }
    return $solde;
  }
   
  /**
   * Retourne le solde point d'un compte donn
   *
   * @param $idCompte identifiant d'un compte, correspond  No_de_compte dans le fichier .gsb
   * @return float le solde point du compte
   */
  public function getSoldePointe($idCompte)
  {
    $solde = '';
    foreach($this->doc->comptes[0]->compte as $compte)
    {
      if ($compte->details[0]->no_de_compte[0]->tagData == $idCompte) {
        $solde = $compte->details[0]->solde_dernier_releve[0]->tagData;
      }
    }
    return $solde;
  }
   
  /**
   * Retourne vrai si un compte est sous le solde minimum autoris
   *
   * @param $idCompte identifiant d'un compte, correspond  No_de_compte dans le fichier .gsb
   * @return boolean vrai si le compte est sous le solde minimum autoris, faux sinon
   */
  public function isSousSoldeAutorise($idCompte)
  {
    foreach($this->doc->comptes[0]->compte as $compte)
    {
      if ($compte->details[0]->no_de_compte[0]->tagData == $idCompte) {
        if (intval($this->getSoldeCourant($idCompte)) < intval($compte->details[0]->solde_mini_autorise[0]->tagData)) {
          return true;
        }
      }
    }
    return false;
  }
  
  /**
   * Retourne le libell du tiers en fonction du paramtre (si le paramtre n'est pas prsent, on retourne la liste complte des tiers)
   * @param $idTiers identifiant d'un tiers
   * @return array le libell du tiers
   */
  public function getLibelleTiers($idTiers = 0)
  {
  	$libelleTiers = array();
  	$count = 0;
  	foreach($this->doc->tiers[0]->detail_des_tiers[0]->tiers as $tiers)
    {
    	if ($idTiers == 0 || $tiers->tagAttrs['no'] == $idTiers)
    	{
    		$libelleTiers[$count]['nom'] = $tiers->tagAttrs['nom'];
    		$libelleTiers[$count]['id'] = $tiers->tagAttrs['no'];
    		$count++;
    	}
    }
  	return $libelleTiers;
  }

  /**
   * Retourne un tableau avec la liste des oprations d'un compte donn
   * @param $idCompte identifiant d'un compte, correspond  No_de_compte dans le fichier .gsb
   * @param $params tableau de paramtres
   * @param $nbOperations nombre d'oprations  retourner
   * @return array tableau avec la liste des oprations
   */
  public function getOperations($idCompte, $params, $nbOperations)
  {
    $operations = array();
    foreach($this->doc->comptes[0]->compte as $compte)
    {
      if ($compte->details[0]->no_de_compte[0]->tagData == $idCompte) {
        $compteur = 0;
        foreach($compte->detail_des_operations[0]->operation as $operation)
        {
        	$display = false;
        	// On parcourt $params. Si les conditions sont runies, on affiche la ligne
        	// TODO voir pour faire des OU au lieu des ET sur la recherche
        	if (!empty($params)) {
        		
            // Traitement de la plage de date
            $dateOperation = convertirTimestamp($operation->tagAttrs['d']);
            $borneDebut = convertirTimestamp($params['dateDebut']);
            $borneFin = convertirTimestamp($params['dateFin']);
            if ($dateOperation >= $borneDebut && $dateOperation <= $borneFin)
            {
            	$display = true;
            }
        		
            // Traitement des autres paramtres
	        	foreach($params as $parametre => $valeur)
	        	{
	        		if ($parametre != 'dateDebut' && $parametre != 'dateFin')
	        		{
		        		if ($operation->tagAttrs[$parametre] == $valeur) {
		        			$display = true;
		        		}
		        		else {
		        			$display = false;
		        			break;
		        		}
	        		}
	        	}
        	}
        	else {
        		$display = true;
        	}
        	
        	if ($display && $compteur < $nbOperations) {
        		// ici, on ne rcupre qu'un seul tiers, car on passe un identifiant
        		$tiers = $this->getLibelleTiers($operation->tagAttrs['t']);
        		
	          $operations[$compteur]['date'] = $operation->tagAttrs['d'];
	          $operations[$compteur]['tiers'] = $tiers[0]['nom'];
	          $operations[$compteur]['cat'] = $this->getCategorie($operation->tagAttrs['c'], $operation->tagAttrs['sc']);
	          $operations[$compteur]['remarque'] = $operation->tagAttrs['n'];
	          $operations[$compteur]['montant'] = $operation->tagAttrs['m'];
            $operations[$compteur]['num_ope_mere'] = $operation->tagAttrs['no'];
            $operations[$compteur]['num_ope_fille'] = $operation->tagAttrs['va'];
	          $compteur++;
        	}
        }
      }
    }
    return $operations;
  }
    
  /**
   * Retourne la catgorie et sous catgorie pour des identifiants donns
   * @param $idCategorie identifiant de la catgorie
   * @param $idSousCategorie identifiant de la sous-catgorie
   * @return array tableau avec la catgorie et le cas chant la sous-catgorie
   */
  public function getCategorie($idCategorie, $idSousCategorie = 0)
  {
    $libelles = array();
    foreach($this->doc->categories[0]->detail_des_categories[0]->categorie as $categorie)
    {
      if ($categorie->tagAttrs['no'] == $idCategorie) {
        $libelles['categorie'] = $categorie->tagAttrs['nom'];
        // parcours des sous-catgories de la catgorie
        if ($idSousCategorie != 0) {
          foreach($categorie->tagChildren as $souscategorie)
          {
            if ($souscategorie->tagAttrs['no'] == $idSousCategorie) {
              $libelles['souscategorie'] = $souscategorie->tagAttrs['nom'];
            }
          }
        }
      }
      $compteur++;
    }
    return $libelles;
  }
  
  public function getCategories()
  {
  	
  }
  
} 
?>
